type Enumerate<N extends number, Acc extends number[] = []> = Acc["length"] extends N ? Acc[number] : Enumerate<N, [...Acc, Acc["length"]]>;
type IntRange<F extends number, T extends number> = F extends T ? F : Exclude<Enumerate<T>, Enumerate<F>> extends never ? never : Exclude<Enumerate<T>, Enumerate<F>> | T;
export type StatusCode1xx = IntRange<100, 199>;
export type StatusCode2xx = IntRange<200, 299>;
export type StatusCode3xx = IntRange<300, 399>;
export type StatusCode4xx = IntRange<400, 499>;
export type StatusCode5xx = IntRange<500, 599>;
export type StatusCode = StatusCode1xx | StatusCode2xx | StatusCode3xx | StatusCode4xx | StatusCode5xx;
export type Res<Status extends StatusCode, Type> = {
    status: Status;
    json: () => Promise<Type>;
};
export type TypedFetchResponse<R extends Res<StatusCode, unknown>> = Promise<Omit<Response, "json" | "status"> & R>;
export interface AbstractFetchClientConfig {
    basePath: string;
    defaultHeaders: Record<string, string>;
    defaultTimeout?: number;
}
export type QueryParams = {
    [name: string]: string | number | number[] | boolean | string[] | undefined | null | QueryParams | QueryParams[];
};
export type HeaderParams = Record<string, string | number | undefined | null> | [string, string | number | undefined | null][] | Headers;
export type HeadersInit = string[][] | readonly (readonly [string, string])[] | Record<string, string | ReadonlyArray<string>> | Headers;
export declare abstract class AbstractFetchClient {
    protected readonly basePath: string;
    protected readonly defaultHeaders: Record<string, string>;
    protected readonly defaultTimeout: number | undefined;
    protected constructor(config: AbstractFetchClientConfig);
    protected _fetch<R extends Res<StatusCode, unknown>>(url: string, opts: RequestInit, timeout?: number | undefined): TypedFetchResponse<R>;
    protected _query(params: QueryParams): string;
    /**
     * Combines headers for a request, with precedence
     * 1. default headers
     * 2. route level header parameters
     * 3. raw request config (escape hatch)
     *
     * following these rules:
     * - header values of `undefined` are skipped
     * - header values of `null` will remove/delete any previously set headers
     *
     * Eg:
     * Passing `Authorization: null` as a parameter, will clear out any
     * default `Authorization` header.
     *
     * But passing `Authorization: undefined` as parameter will fallthrough
     * to the default `Authorization` header.
     *
     * @param paramHeaders
     * @param optsHeaders
     * @protected
     */
    protected _headers(paramHeaders?: HeaderParams, optsHeaders?: HeadersInit): Headers;
    private setHeaders;
    private headersAsArray;
}
export {};
